# This code is a part of Slash, and is released under the GPL.
# Copyright 1997-2001 by Open Source Development Network. See README
# and COPYING for more information, or see http://slashcode.com/.

# This file has seen modifications to turn this file, which was at one point a Perl CGI script into a Apache mod_perl2 module handler. This file and code in it is derived from works of the original authors and copyright owners (Open Source Developers Network). The software package "Slashcode" was released as GPL. This gives others the legal right to use, modify, and redistribute modified versions of Slashcode.

# Code was added to the GPL'ed version of what OSDN has released at version 2.2.6. Namely, code was added so that users client browsers can authenticated using passwords and athentication data stored in HTTP cookies. These modifications are copyright 2014 by Troye "taj" Johnson <troyejohnson@yandex.com>. The license for these modifications is GPLv2, or later.

# note: requires, for now, Mozilla 0.9 (not Mozilla 0.9.1!) and
# ForumZilla 0.3.1.  See http://forumzilla.mozdev.org/.

package slash_fz_pl; # for /fz.pl

use strict;

use Slash 2.001;    # require Slash 2.1
use Slash::Display;
use Slash::Utility;
use Slash::XML;

use Apache2::Const qw(REDIRECT OK);

use vars qw($VERSION);

($VERSION) = ' $Revision: 1.1.2.4 $ ' =~ /\$Revision:\s+([^\s]+)/;

use constant ALLOWED  => 0;
use constant FUNCTION => 1;

# check if $ENV{SLASH_VIRTUAL_USER} and $ENV{SLASH_CONFIG} defined in httpd.conf
slash_preinit_check();

# comes from httpd.conf
my $VIRTUAL_USER = $ENV{SLASH_VIRTUAL_USER};

# must be called before handler() is run
slash_api_init($VIRTUAL_USER);


=item slash_index_pl::handler

Resides in: slash_index_pl.pm

This handler is for the /index.pl URL. Since this is a modperl handler, the '<Location></Location>' tags will have to set the URL, so that Apache will map the URL to the proper module handler in order for the request to be served.

Most of these params will not be handled by the handler() or interpret their meaning. main() will handle that stuff.

All handler does is some pre-processing to set-up the main SLASH_STRUCTS (you can fetch these with getCurrentUser, getCurrentForm, and of course getCurrentDB, most or all of the getCurrent*() subs are used to fetch so call "SLASH_STRUCTS". They are just HASH references used by the API.) so that the handler() can call main(), and thus the module handler has done it's job.

Argument list for this handler. "op" seems like the /index.pl links back to itself so that the site can re-display the page. "section" seems to also refer back to the site to display things.

USER_STRUCT; needed

=cut

sub handler {
    my ( $r ) = @_;

    die "slash_users_pl: slash_api_init() not called"
	unless SLASH_API_INIT_OK;

    my $uid_Param = undef; # This available after authentication process

    my $form_Param = slash_get_params( $r );
    my $cookie_args_Param = slash_get_cookies( $r );
    my $uri_Param = $r->uri(); # e.g, /index.html, /index.pl, or /foo.(html|pl)

    my $method_Param = $r->the_request(); $method_Param =~ s/^(\S*)/$1/;
    $method_Param = $1; # should now be either GET, POST

    my ($unickname, $upasswd); # params from form
    $unickname = $form_Param->{unickname} if $form_Param->{unickname};
    $upasswd = $form_Param->{upasswd} if $form_Param->{upasswd};

    #
    # Attempt to authenticate user
    #

    # $auth_type: 1 is for form login. 2 is for authentication via cookie. Set this to "2" as /users.pl is the only script which can auth w/ "auth_type = 1" -- which is for username and password logins only. This script can authenticate only using cookies.
    my $auth_type = 2;
 
    my $authing_user = undef; # uid of authenticated user, else AnonymousCoward
    my $auth_success = 0; # 1 for successful auth, 0 for authentication failure
    my $auth_uid_field = 0; # First element in array returned by Slash::DB::gutUserAuthenticate()
      
    # For cookie authentication: authenticate with cookie
    if ($auth_type == 2) {
	( $unickname, $upasswd ) = eatUserCookie($cookie_args_Param->{users});

	$authing_user = slash_web_auth( $unickname, $upasswd, 2);
	$auth_success = 1 if $authing_user;

	# `logger 'handler: auth type => 2'`; 
	# `logger 'handler: users cookie in val => '$cookie_args->{users}''`;
	# `logger 'handler: eatUserCookie: n => $unickname, p => $upasswd'`; 
    }
    
    
    if ($auth_success == 1) {
	$uid_Param = $authing_user->[$auth_uid_field];    
    }

    # ** DISABLED: this code designed to follow up after a "auth_type = 1" login
    #
    # if ( ($auth_success == 1) && ($auth_type == 1) ) {
    # 	# ** Set cookie if auth_type is login
	
    # 	# Does not matter whether $upasswd is encrypted or not
    # 	my $users_cookie_data
    # 	    = bakeUserCookie( $authing_user->[$auth_uid_field], $upasswd);
    # 	setCookie('users', $users_cookie_data);
	
    # 	# `logger 'users cookie out value => '$users_cookie_data''`;
    # 	# `logger 'handler: auth was good'`;
    # }
    

    # If User did not authenticate, then they will be anonymous
    if (! defined $uid_Param) {
	$uid_Param = SLASH_ANON_COWARD_UID();
	# `logger 'handler: auth not successful'`; 
    }

    createEnvironment($VIRTUAL_USER, $form_Param, $uid_Param, $uri_Param, $cookie_args_Param, $method_Param);
    main();
    return Apache2::Const::OK;
}


sub main {
    my $slashdb   = getCurrentDB();       #getObject('Slash::ForumZilla');
    my $constants = getCurrentStatic();
    my $user      = getCurrentUser();
    my $form      = getCurrentForm();

    # possible value of "op" parameter in form
    my %ops = (
        ad     => [ 1, \&fzDispAd ],
        navbar => [ 1, \&fzDispNavbar ],

        forum          => [ 1, \&fzForum ],
        describeStory  => [ 1, \&fzStory ],
        displayStory   => [ 1, \&fzDispStory ],
        displayComment => [ 1, \&fzDispComment ],

        default => [ 1, \&fzForum ]
    );

    $user->{commentsort} = 0;

    # ack, make sure mode isn't archive ?
    $user->{mode}      = 'thread';
    $user->{threshold} = -1;

    # prepare op to proper value if bad value given
    my $op = $form->{op};
    if ( !$op || !exists $ops{$op} || !$ops{$op}[ALLOWED] ) {
        $op = 'default';
    }

    # dispatch of op
    $ops{$op}[FUNCTION]->( $slashdb, $constants, $user, $form );

    # writeLog('SOME DATA');	# if appropriate
}

sub fzForum {
    my ( $slashdb, $constants, $user, $form ) = @_;
    my $stories = $slashdb->fzGetStories( $form->{section} );

    xmlDisplay(
        'fz',
        {
            stories => $stories,
            section => $form->{section},
            type    => 'forum',
        }
    );
}

sub fzStory {
    my ( $slashdb, $constants, $user, $form ) = @_;
    $user->{state}{noreparent} = 1;
    my ($comments) =
      Slash::selectComments( $slashdb->getDiscussionBySid( $form->{sid} ) );

    xmlDisplay(
        'fz',
        {
            # move to plugin eventually, probably
            comments => $comments,
            sid      => $form->{sid},
            section  => $form->{section},
            type     => 'story',
        }
    );
}

sub fzDispStory {
    my ( $slashdb, $constants, $user, $form ) = @_;

    header();
    my ( $data, $story ) = displayStory( $form->{sid}, 'Full' );
    print $data;

    $user->{state}{nocomment} = 1;
    my $discussion = $slashdb->getDiscussionBySid( $form->{sid} );
    printComments($discussion);

    footer();
}

sub fzDispComment {
    my ( $slashdb, $constants, $user, $form ) = @_;
    $user->{state}{noreparent} = 1;
    my ($comments) =
      Slash::selectComments( $slashdb->getDiscussionBySid( $form->{sid} ) );

    header();
    print dispComment( $comments->{ $form->{cid} } );
    footer();
}

sub fzDispAd {
    my ( $slashdb, $constants, $user, $form ) = @_;
    $user->{is_admin} = 0;

    header('Advertisement');
    print getAd();
    footer();
}

sub fzDispNavbar {
    my ( $slashdb, $constants, $user, $form ) = @_;
    $user->{is_admin} = 0;

    header();
    slashDisplay('mainmenu');
    footer();
}


1;
