# This code is a part of Slash, and is released under the GPL.
# Copyright 1997-2001 by Open Source Development Network. See README
# and COPYING for more information, or see http://slashcode.com/.

# This file has seen modifications to turn this file, which was at one point a Perl CGI script into a Apache mod_perl2 module handler. This file and code in it is derived from works of the original authors and copyright owners (Open Source Developers Network). The software package "Slashcode" was released as GPL. This gives others the legal right to use, modify, and redistribute modified versions of Slashcode.

# Code was added to the GPL'ed version of what OSDN has released at version 2.2.6. Namely, code was added so that users client browsers can authenticated using passwords and athentication data stored in HTTP cookies. These modifications are copyright 2014 by Troye "taj" Johnson <troyejohnson@yandex.com>. The license for these modifications is GPLv2, or later.

package slash_metamod_pl; # for /metamod.pl

use strict;

use Slash;
use Slash::Display;
use Slash::Utility;
use Apache2::Const qw(OK);

# check if $ENV{SLASH_VIRTUAL_USER} and $ENV{SLASH_CONFIG} defined in httpd.conf
slash_preinit_check();

# comes from httpd.conf
my $VIRTUAL_USER = $ENV{SLASH_VIRTUAL_USER};

# must be called before handler() is run
slash_api_init($VIRTUAL_USER);


=item slash_index_pl::handler

Resides in: slash_index_pl.pm

This handler is for the /index.pl URL. Since this is a modperl handler, the '<Location></Location>' tags will have to set the URL, so that Apache will map the URL to the proper module handler in order for the request to be served.

Most of these params will not be handled by the handler() or interpret their meaning. main() will handle that stuff.

All handler does is some pre-processing to set-up the main SLASH_STRUCTS so that the handler() can call main(), and thus the module handler has done it's job.

=cut

sub handler {
    my ( $r ) = @_;

    die "slash_users_pl: slash_api_init() not called"
	unless SLASH_API_INIT_OK;

    my $uid_Param = undef; # This available after authentication process

    my $form_Param = slash_get_params( $r );
    my $cookie_args_Param = slash_get_cookies( $r );
    my $uri_Param = $r->uri(); # e.g, /index.html, /index.pl, or /foo.(html|pl)

    my $method_Param = $r->the_request(); $method_Param =~ s/^(\S*)/$1/;
    $method_Param = $1; # should now be either GET, POST

    my ($unickname, $upasswd); # params from form
    $unickname = $form_Param->{unickname} if $form_Param->{unickname};
    $upasswd = $form_Param->{upasswd} if $form_Param->{upasswd};

    #
    # Attempt to authenticate user
    #

    # $auth_type: 1 is for form login. 2 is for authentication via cookie. Set this to "2" as /users.pl is the only script which can auth w/ "auth_type = 1" -- which is for username and password logins only. This script can authenticate only using cookies.
    my $auth_type = 2;
 
    my $authing_user = undef; # uid of authenticated user, else AnonymousCoward
    my $auth_success = 0; # 1 for successful auth, 0 for authentication failure
    my $auth_uid_field = 0; # First element in array returned by Slash::DB::gutUserAuthenticate()
      
    # For cookie authentication: authenticate with cookie
    if ($auth_type == 2) {
	( $unickname, $upasswd ) = eatUserCookie($cookie_args_Param->{users});

	$authing_user = slash_web_auth( $unickname, $upasswd, 2);
	$auth_success = 1 if $authing_user;

	# `logger 'handler: auth type => 2'`; 
	# `logger 'handler: users cookie in val => '$cookie_args->{users}''`;
	# `logger 'handler: eatUserCookie: n => $unickname, p => $upasswd'`; 
    }
    
    
    if ($auth_success == 1) {
	$uid_Param = $authing_user->[$auth_uid_field];    
    }

    # ** DISABLED: this code designed to follow up after a "auth_type = 1" login
    #
    # if ( ($auth_success == 1) && ($auth_type == 1) ) {
    # 	# ** Set cookie if auth_type is login
	
    # 	# Does not matter whether $upasswd is encrypted or not
    # 	my $users_cookie_data
    # 	    = bakeUserCookie( $authing_user->[$auth_uid_field], $upasswd);
    # 	setCookie('users', $users_cookie_data);
	
    # 	# `logger 'users cookie out value => '$users_cookie_data''`;
    # 	# `logger 'handler: auth was good'`;
    # }
    

    # If User did not authenticate, then they will be anonymous
    if (! defined $uid_Param) {
	$uid_Param = SLASH_ANON_COWARD_UID();
	# `logger 'handler: auth not successful'`; 
    }

    createEnvironment($VIRTUAL_USER, $form_Param, $uid_Param, $uri_Param, $cookie_args_Param, $method_Param);
    main();
    return Apache2::Const::OK;
}

#################################################################
sub main {
    my $slashdb   = getCurrentDB();
    my $constants = getCurrentStatic();
    my $user      = getCurrentUser();
    my $form      = getCurrentForm();
    my $op        = getCurrentForm('op');

    header( getData('header') );

    my $ineligible =
         $user->{is_anon}
      || $user->{rtbl}
      || !$slashdb->checkForMetaModerator($user);

    if ( !$constants->{allow_moderation} ) {
        print getData('no_moderation');
    }
    elsif ($ineligible) {
        print getData('not-eligible');
    }
    else {
        #my $last =
        #	($slashdb->getModeratorLast($user->{uid}))->{lastmmid};
        #unless ($last) {
        #$last = $slashdb->getModeratorLogRandom($uid);
        #$slashdb->setUser($user->{uid}, {
        #	lastmmid => $last,
        #});
        #}

        if ( $op eq 'MetaModerate' ) {
            metaModerate();
        }
        else {
            displayTheComments();
        }
    }

    writeLog($op);
    footer();
}

#################################################################
# This is deprecated and not used in this scope.
# 	- Cliff 08/24/01
#
#sub karmaBonus {
#	my $constants = getCurrentStatic();
#	my $user = getCurrentUser();
#
#	my $x = $constants->{m2_maxbonus} - $user->{karma};
#
#	return 0 unless $x > 0;
#	return 1 if rand($constants->{m2_maxbonus}) < $x;
#	return 0;
#}

#################################################################
sub metaModerate {
    my ($id)      = @_;
    my $slashdb   = getCurrentDB();
    my $constants = getCurrentStatic();
    my $user      = getCurrentUser();
    my $form      = getCurrentForm();

    my ( %metamod, @mmids );
    my $y = $constants->{m2_comments};
    $metamod{unfair} = $metamod{fair} = 0;
    for ( keys %{$form} ) {

        # Meta mod form data can only be a '+' or a '-' so we apply some
        # protection from taint.
        next if $form->{$_} !~ /^[+-]$/;    # bad input, bad!
        if (/^mm(\d+)$/) {

            # Sanity check. If this isn't right, then someone's
            # fiddling with the form.
            if ( $y < 1 ) {
                print getData('unexpected_item');
                return;
            }
            push( @mmids, $1 ) if $form->{$_};
            $metamod{unfair}++ if $form->{$_} eq '-';
            $metamod{fair}++   if $form->{$_} eq '+';
            $y--;
        }
    }

    my %m2victims;
    for (@mmids) {
        $m2victims{$_} =
          [ $slashdb->getModeratorLog( $_, 'uid' ), $form->{"mm$_"} ];
    }

    # Obsoleted by consensus moderation:
    #	vars.m2_mincheck, vars.m2_maxunfair, vars.m2_toomanyunfair
    # M2 validation is now determined by long term analysis. This analysis
    # is now left to the slashd moderation Task, which can vary by theme.
    #
    # Note the use of a naked "10" here for the M2 flag. This is used
    # to denote M2 entries that have yet to be reconciled.
    my $changes = $slashdb->setMetaMod( \%m2victims, 10, scalar time );

    slashDisplay(
        'metaModerate',
        {
            changes => $changes,
            count   => $constants->{m2_comments} - $y,
            metamod => \%metamod,
        }
    );

    $slashdb->setModeratorVotes( $user->{uid}, \%metamod );
}

#################################################################
sub displayTheComments {
    my ($id)      = @_;
    my $slashdb   = getCurrentDB();
    my $constants = getCurrentStatic();
    my $user      = getCurrentUser();

    my $comments =
      $slashdb->getMetamodComments( $user, $constants->{m2_comments} );

    # We set this to prevent the "Reply" and "Parent" links from
    # showing up. If the metamoderator needs context, they can use
    # the CID link.
    $user->{mode} = 'archive';

    slashDisplay(
        'dispTheComments',
        {
            comments => $comments,
        }
    );
}

#################################################################
# This is going to break under replication
#
# Yeah, this should be a lot more flexible, but lets leave that
# issue for when we revamp security. For now, we'll just use
# $slashdb->checkForMetaModerator(). What we do lose is the
# reporting behind WHY a user can't M2, which isn't all that
# critical right now, since they will get an error of some sort.
#						- Cliff
#sub isEligible {
#	my $slashdb = getCurrentDB();
#	my $constants = getCurrentStatic();
#	my $user = getCurrentUser();
#
#	my $tuid = $slashdb->countUsers();
#	my $last = $slashdb->getModeratorLast($user->{uid});
#
#	my $result = slashDisplay('isEligible', {
#		user_count	=> $tuid,
#		'last'		=> $last,
#	}, { Return => 1, Nocomm => 1 });
#
#	if ($result ne 'Eligible') {
#		print $result;
#		return 0;
#	}
#
#	# Eligible for M2. Determine M2 comments by selecting random starting
#	# point in moderatorlog.
#	unless ($last->{'lastmmid'}) {
#		$last->{'lastmmid'} = $slashdb->getModeratorLogRandom();
#		$slashdb->setUser($user->{uid}, { lastmmid => $last->{'lastmmid'} });
#	}
#
#	return $last->{'lastmmid'}; # Hooray!
#}


1;

