# This code is a part of Slash, and is released under the GPL.
# Copyright 1997-2001 by Open Source Development Network. See README
# and COPYING for more information, or see http://slashcode.com/.

# This file has seen modifications to turn this file, which was at one point a Perl CGI script into a Apache mod_perl2 module handler. This file and code in it is derived from works of the original authors and copyright owners (Open Source Developers Network). The software package "Slashcode" was released as GPL. This gives others the legal right to use, modify, and redistribute modified versions of Slashcode.

# Code was added to the GPL'ed version of what OSDN has released at version 2.2.6. Namely, code was added so that users client browsers can authenticated using passwords and athentication data stored in HTTP cookies. These modifications are copyright 2014 by Troye "taj" Johnson <troyejohnson@yandex.com>. The license for these modifications is GPLv2, or later.

package slash_sections_pl; # for /sections.pl

use strict;

use Slash;
use Slash::Display;
use Slash::Utility;

use Apache2::Const qw(OK);

# check if $ENV{SLASH_VIRTUAL_USER} and $ENV{SLASH_CONFIG} defined in httpd.conf
slash_preinit_check();

# comes from httpd.conf
my $VIRTUAL_USER = $ENV{SLASH_VIRTUAL_USER};

# must be called before handler() is run
slash_api_init($VIRTUAL_USER);


=item slash_index_pl::handler

Resides in: slash_index_pl.pm

This handler is for the /index.pl URL. Since this is a modperl handler, the '<Location></Location>' tags will have to set the URL, so that Apache will map the URL to the proper module handler in order for the request to be served.

Most of these params will not be handled by the handler() or interpret their meaning. main() will handle that stuff.

All handler does is some pre-processing to set-up the main SLASH_STRUCTS so that the handler() can call main(), and thus the module handler has done it's job.

USER_STRUCT; yes needed

=cut

sub handler {
    my ( $r ) = @_;

    die "slash_users_pl: slash_api_init() not called"
	unless SLASH_API_INIT_OK;

    my $uid_Param = undef; # This available after authentication process

    my $form_Param = slash_get_params( $r );
    my $cookie_args_Param = slash_get_cookies( $r );
    my $uri_Param = $r->uri(); # e.g, /index.html, /index.pl, or /foo.(html|pl)

    my $method_Param = $r->the_request(); $method_Param =~ s/^(\S*)/$1/;
    $method_Param = $1; # should now be either GET, POST

    my ($unickname, $upasswd); # params from form
    $unickname = $form_Param->{unickname} if $form_Param->{unickname};
    $upasswd = $form_Param->{upasswd} if $form_Param->{upasswd};

    #
    # Attempt to authenticate user
    #

    # $auth_type: 1 is for form login. 2 is for authentication via cookie. Set this to "2" as /users.pl is the only script which can auth w/ "auth_type = 1" -- which is for username and password logins only. This script can authenticate only using cookies.
    my $auth_type = 2;
 
    my $authing_user = undef; # uid of authenticated user, else AnonymousCoward
    my $auth_success = 0; # 1 for successful auth, 0 for authentication failure
    my $auth_uid_field = 0; # First element in array returned by Slash::DB::gutUserAuthenticate()
      
    # For cookie authentication: authenticate with cookie
    if ($auth_type == 2) {
	( $unickname, $upasswd ) = eatUserCookie($cookie_args_Param->{users});

	$authing_user = slash_web_auth( $unickname, $upasswd, 2);
	$auth_success = 1 if $authing_user;

	# `logger 'handler: auth type => 2'`; 
	# `logger 'handler: users cookie in val => '$cookie_args->{users}''`;
	# `logger 'handler: eatUserCookie: n => $unickname, p => $upasswd'`; 
    }
    
    
    if ($auth_success == 1) {
	$uid_Param = $authing_user->[$auth_uid_field];    
    }

    # ** DISABLED: this code designed to follow up after a "auth_type = 1" login
    #
    # if ( ($auth_success == 1) && ($auth_type == 1) ) {
    # 	# ** Set cookie if auth_type is login
	
    # 	# Does not matter whether $upasswd is encrypted or not
    # 	my $users_cookie_data
    # 	    = bakeUserCookie( $authing_user->[$auth_uid_field], $upasswd);
    # 	setCookie('users', $users_cookie_data);
	
    # 	# `logger 'users cookie out value => '$users_cookie_data''`;
    # 	# `logger 'handler: auth was good'`;
    # }
    

    # If User did not authenticate, then they will be anonymous
    if (! defined $uid_Param) {
	$uid_Param = SLASH_ANON_COWARD_UID();
	# `logger 'handler: auth not successful'`; 
    }

    createEnvironment($VIRTUAL_USER, $form_Param, $uid_Param, $uri_Param, $cookie_args_Param, $method_Param);
    main();
    return Apache2::Const::OK;
}


#################################################################
sub main {
    my $user = getCurrentUser();
    my $form = getCurrentForm();

    my $op     = $form->{op};
    my $seclev = $user->{seclev};

    if ( $seclev < 100 ) {
        my $rootdir = getCurrentStatic('rootdir');
        redirect("$rootdir/users.pl");
        return;
    }

    header( getData('head'), 'admin' );

    if ( $op eq 'rmsub' && $seclev >= 100 ) {    # huh?

    }
    elsif ( $form->{addsection} ) {
        titlebar( '100%', getData('addhead') );
        editSection();

    }
    elsif ($form->{deletesection}
        || $form->{deletesection_cancel}
        || $form->{deletesection_confirm} )
    {
        delSection( $form->{section} );
        listSections($user);

    }
    elsif ( $op eq 'editsection' || $form->{editsection} ) {
        titlebar( '100%', getData('edithead') );
        editSection( $form->{section} );

    }
    elsif ( $form->{savesection} ) {
        titlebar( '100%', getData('savehead') );
        saveSection( $form->{section} );
        listSections($user);

    }
    elsif ( ( !defined $op || $op eq 'list' ) && $seclev >= 500 ) {
        titlebar( '100%', getData('listhead') );
        listSections($user);
    }

    footer();
}

#################################################################
sub listSections {
    my ($user) = @_;
    my $slashdb = getCurrentDB();

    if ( $user->{section} ) {
        editSection( $user->{section} );
        return;
    }

    slashDisplay(
        'listSections',
        {
            sections => $slashdb->getSectionTitle()
        }
    );
}

#################################################################
sub delSection {
    my ($section) = @_;
    my $slashdb   = getCurrentDB();
    my $form      = getCurrentForm();

    if ( $form->{deletesection} ) {
        slashDisplay(
            'delSection',
            {
                section => $section
            }
        );
    }
    elsif ( $form->{deletesection_cancel} ) {
        slashDisplay(
            'delSectCancel',
            {
                section => $section
            }
        );
    }
    elsif ( $form->{deletesection_confirm} ) {
        slashDisplay(
            'delSectConfirm',
            {
                section => $section,
                title   => "Deleted $section Section",
                width   => '100%'
            }
        );
        $slashdb->deleteSection($section);
    }
}

#################################################################
sub editSection {
    my ($section) = @_;
    my $slashdb   = getCurrentDB();
    my $user      = getCurrentUser();
    my $form      = getCurrentForm();

    my ( @blocks, $this_section );
    if ( $form->{addsection} ) {
        $this_section = {};
    }
    else {
        $this_section = $slashdb->getSection($section);
        my $blocks = $slashdb->getSectionBlock($section);

        for (@$blocks) {
            my $block = $blocks[@blocks] = {};
            @{$block}{qw(section bid ordernum title portal url)} = @$_;
            $block->{title} =~ s/<(.*?)>//g;

        }
    }

    my $qid = createSelect( 'qid', $slashdb->getPollQuestions(),
        $this_section->{qid}, 1 );
    my $isolate = createSelect(
        'isolate',
        $slashdb->getDescriptions('isolatemodes'),
        $this_section->{isolate}, 1
    );
    my $issue = createSelect(
        'issue',
        $slashdb->getDescriptions('issuemodes'),
        $this_section->{issue}, 1
    );

    slashDisplay(
        'editSection',
        {
            section      => $section,
            this_section => $this_section,
            qid          => $qid,
            isolate      => $isolate,
            issue        => $issue,
            blocks       => \@blocks,
            topics       => $slashdb->getSectionTopicsNamesBySection($section),
        }
    );
}

#################################################################
sub saveSection {
    my ($section) = @_;
    my $slashdb   = getCurrentDB();
    my $form      = getCurrentForm();

    # Non alphanumerics are not allowed in the section key.
    # And I don't see a reason for underscores either, but
    # dashes should be allowed.
    $section =~ s/[^A-Za-z0-9\-]//g;

    my ( $count, $ok1, $ok2 ) = $slashdb->setSection(
        @{$form}{qw(section qid title issue isolate artcount)} );

    unless ($ok1) {
        print getData( 'insert', { section => $section } );
    }

    unless ($ok2) {
        print getData( 'update', { section => $section } );
    }
}



1;
